@@ -1737,8 +1737,8 @@ sse_rule_splitql (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest2 = p->vars[insn->dest_args[1]].alloc;
 
 #ifndef MMX
-  orc_sse_emit_pshufd (p, ORC_SSE_SHUF(2,0,2,0), src, dest2);
   orc_sse_emit_pshufd (p, ORC_SSE_SHUF(3,1,3,1), src, dest1);
+  orc_sse_emit_pshufd (p, ORC_SSE_SHUF(2,0,2,0), src, dest2);
 #else
   orc_sse_emit_movdqa (p, src, dest2);
   orc_sse_emit_pshufw (p, ORC_SSE_SHUF(3,2,3,2), src, dest1);
@@ -1757,9 +1757,8 @@ sse_rule_splitlw (OrcCompiler *p, void *user, OrcInstruction *insn)
   orc_sse_emit_psrad_imm (p, 16, dest1);
   orc_sse_emit_packssdw (p, dest1, dest1);
 
-  if (dest2 != src) {
+  if (dest2 != src)
     orc_sse_emit_movdqa (p, src, dest2);
-  }
   orc_sse_emit_pslld_imm (p, 16, dest2);
   orc_sse_emit_psrad_imm (p, 16, dest2);
   orc_sse_emit_packssdw (p, dest2, dest2);
@@ -1774,14 +1773,14 @@ sse_rule_splitwb (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest2 = p->vars[insn->dest_args[1]].alloc;
   int tmp = orc_compiler_get_constant (p, 2, 0xff);
 
+  ORC_DEBUG ("got tmp %d", tmp);
   /* FIXME slow */
 
   orc_sse_emit_psraw_imm (p, 8, dest1);
   orc_sse_emit_packsswb (p, dest1, dest1);
 
-  if (dest2 != src) {
+  if (dest2 != src)
     orc_sse_emit_movdqa (p, src, dest2);
-  }
 
 #if 0
   orc_sse_emit_psllw_imm (p, 8, dest2);
@@ -1957,6 +1956,51 @@ sse_rule_swapq_ssse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
   }
 }
 
+static void
+sse_rule_splitlw_ssse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest1 = p->vars[insn->dest_args[0]].alloc;
+  int dest2 = p->vars[insn->dest_args[1]].alloc;
+  int tmp1, tmp2;
+
+  tmp1 = orc_compiler_try_get_constant_long (p,
+      0x07060302, 0x0f0e0b0a, 0x07060302, 0x0f0e0b0a);
+  tmp2 = orc_compiler_try_get_constant_long (p,
+      0x05040100, 0x0d0c0908, 0x05040100, 0x0d0c0908);
+  if (tmp1 != ORC_REG_INVALID && tmp2 != ORC_REG_INVALID) {
+    orc_sse_emit_pshufb (p, tmp1, dest1);
+    if (dest2 != src)
+      orc_sse_emit_movdqa (p, src, dest2);
+    orc_sse_emit_pshufb (p, tmp2, dest2);
+  } else {
+    sse_rule_splitlw (p, user, insn);
+  }
+}
+
+
+static void
+sse_rule_splitwb_ssse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src = p->vars[insn->src_args[0]].alloc;
+  int dest1 = p->vars[insn->dest_args[0]].alloc;
+  int dest2 = p->vars[insn->dest_args[1]].alloc;
+  int tmp1, tmp2;
+
+  tmp1 = orc_compiler_try_get_constant_long (p,
+      0x07050301, 0x0f0d0b09, 0x07050301, 0x0f0d0b09);
+  tmp2 = orc_compiler_try_get_constant_long (p,
+      0x06040200, 0x0e0c0a08, 0x06040200, 0x0e0c0a08);
+  if (tmp1 != ORC_REG_INVALID && tmp2 != ORC_REG_INVALID) {
+    orc_sse_emit_pshufb (p, tmp1, dest1);
+    if (dest2 != src)
+      orc_sse_emit_movdqa (p, src, dest2);
+    orc_sse_emit_pshufb (p, tmp2, dest2);
+  } else {
+    sse_rule_splitwb (p, user, insn);
+  }
+}
+
 static void
 sse_rule_select0lw_ssse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -2933,6 +2977,8 @@ orc_compiler_sse_register_rules (OrcTarget *target)
   orc_rule_register (rule_set, "swapl", sse_rule_swapl_ssse3, NULL);
   orc_rule_register (rule_set, "swapwl", sse_rule_swapwl_ssse3, NULL);
   orc_rule_register (rule_set, "swapq", sse_rule_swapq_ssse3, NULL);
+  orc_rule_register (rule_set, "splitlw", sse_rule_splitlw_ssse3, NULL);
+  orc_rule_register (rule_set, "splitwb", sse_rule_splitwb_ssse3, NULL);
   orc_rule_register (rule_set, "select0lw", sse_rule_select0lw_ssse3, NULL);
   orc_rule_register (rule_set, "select1lw", sse_rule_select1lw_ssse3, NULL);
   orc_rule_register (rule_set, "select0wb", sse_rule_select0wb_ssse3, NULL);
